/*
 * Generic pager functionality.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <systypes/base.h>

#include <fsserver/page_mapper.h>
#include <fsserver/resource.h>
#include <mem/memory_incremental.h>



/* A pager exposing a dataspace. */

class Pager : public Resource
{
protected:
    offset_t _start, _size;
    PageMapper *_mapper;
    map_flags_t _flags;

    /* Masked region support. */

    bool _is_masked;
    offset_t _start_visible, _end_visible;
    Flexpage _start_flexpage, _end_flexpage, _zero_flexpage;
    MemoryIncremental _memory;

    Flexpage *get_masked_flexpage(Flexpage *flexpage);

    void allocate_region(Flexpage *flexpage, Flexpage &masked);

    void populate_region(Flexpage *flexpage, Flexpage &masked,
                         bool has_start, bool has_end);

public:
    explicit Pager(PageMapper *mapper, map_flags_t flags);

    virtual void close();

    /* Paging methods. */

    virtual long map(offset_t offset, address_t hot_spot, map_flags_t flags,
                     l4_snd_fpage_t *region);

    /* Limit methods. */

    offset_t get_data_size();

    /* File methods. */

    virtual long flush(offset_t populated_size, offset_t *size);

    virtual long resize(offset_t *size);

    /* Mapped file methods. */

    virtual long mmap(offset_t position, offset_t length, int masked,
                      offset_t *start_pos, offset_t *end_pos, offset_t *size);
};

// vim: tabstop=4 expandtab shiftwidth=4
